/*
  Copyright (c) 2015-2019 Montel Laurent <montel@kde.org>

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef CUSTOMTOOLSVIEWINTERFACE_H
#define CUSTOMTOOLSVIEWINTERFACE_H

#include "pimcommon_export.h"

#include <QWidget>
class KToggleAction;
namespace PimCommon {
/**
 * @brief The CustomToolsViewInterface class
 * @author Laurent Montel <montel@kde.org>
 */
class PIMCOMMON_EXPORT CustomToolsViewInterface : public QWidget
{
    Q_OBJECT
public:
    explicit CustomToolsViewInterface(QWidget *parent = nullptr);
    ~CustomToolsViewInterface() override;

    virtual void setText(const QString &text);
    virtual KToggleAction *action() const;

Q_SIGNALS:
    void insertText(const QString &text);
    void toolsWasClosed();
    void activateView(QWidget *);
};
}
#endif // CUSTOMTOOLSVIEWINTERFACE_H
